package  
{
	import com.greensock.TweenLite;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.media.Sound;
	import flash.ui.Keyboard;
	
	/**
	 * ...
	 * @author 
	 */
	public class Game extends Sprite 
	{
		public static const W:int = 600;
		public static const H:int = 600;
		private var _world:World;
		private var _levelGen:LevelGen = new LevelGen();
		private var _gen:int = 1;
		
		[Embed(source='ambient.mp3')] 		 
		private var Music:Class; 		 
		private var _waiting:Boolean = false;
		
		private var _menu:Menu = null;
		private var _generation:Generation;
		
		public function Game() 
		{
			_resetLevel();
			addEventListener(Event.ADDED_TO_STAGE, _onAddedToStage);

			const music:Sound = new Music;
			music.play(0, 10000);
			_openMenu();
			
			_generation = new Generation();
			_generation.x = W - _generation.width;
			_generation.y = H - _generation.height;
			_generation.tf.text = "";
			addChild(_generation);
			_generation.alpha = 0.5;
		}
		
		private function _onAddedToStage(e:Event):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, _onAddedToStage);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, _onKey);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, _onMouseDown);
		}
		
		private function _onMouseDown(e:MouseEvent):void 
		{
			_closeMenu();
		}
		
		private function _onKey(e:KeyboardEvent):void 
		{
			if (e.keyCode == Keyboard.BACKSPACE && CONFIG::debug)
			{
				++_gen;
				_resetLevel();
			}
			else if (e.keyCode == Keyboard.ESCAPE)
			{
				if (_menu == null)
					_openMenu();
				else
					_closeMenu();
			}
			else if (e.keyCode == 82) // R
			{
				_gen = 1;
				_resetLevel();
			}
			else if (e.keyCode == Keyboard.SPACE)
			{
				_resetLevel();
			}
		}
		
		public function tick(tick:int):void
		{
			_generation.tf.text = _gen.toString();
			
			_world.tick(tick);
			
			setChildIndex(_generation, numChildren - 1);
			if (_menu != null)
			{
				setChildIndex(_menu, numChildren-1);
				return;
			}
			if (_waiting)
				return;
			if (_world.isComplete())
			{
				++_gen;
				_resetLevel();
			} else if (_world.hasFailed())
			{
				_resetLevel();
			}
		}
		
		public function render(tick:int, a:Number):void
		{
			_world.render(tick, a);
		}
		
		private function _resetLevel():void
		{
			if (_world == null)
				_replaceWorld();
			else
			{
				TweenLite.to(_world, 2, { alpha:0, onComplete: _replaceWorld } );
				_waiting = true;
			}
		}
		
		private function _replaceWorld():void
		{
			_waiting = false;
			if (_world != null && contains(_world))
				removeChild(_world);
			_world = _levelGen.generate(_gen);
			_world.alpha = 0;
			addChild(_world);
			TweenLite.to(_world, 2, { alpha:1 } );
		}
		
		private function _openMenu():void 
		{
			if (_menu != null)
				return;
			_menu = new Menu();
			_menu.x = (W - _menu.width) * 0.5;
			_menu.y = (H - _menu.height) * 0.5 + 50;
			_menu.alpha = 0.6;
			addChild(_menu);
		}
		
		private function _closeMenu():void
		{
			if (_menu == null)
				return;
			removeChild(_menu);
			_menu = null;
		}
	}
}