package  
{
	import entity.Entity;
	import entity.HumanOrganism;
	import entity.Organism;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import ui.SelectionMarker;
	import util.Vec2;
	/**
	 * ...
	 * @author 
	 */
	public class World extends Sprite
	{
		private const _entities:Vector.<Entity> = new Vector.<Entity>();
		private const _selectionMarker:SelectionMarker = new SelectionMarker;
		private var _selected:HumanOrganism;
		private var _isComplete:Boolean = false;
		private var _failed:Boolean = false;
		public var _totalOrgs:uint = 0;
		
		public function World() 
		{
			graphics.beginFill(0x0, 1);
			graphics.drawRect(0, 0, Game.W, Game.H);
			graphics.endFill();
			
			addEventListener(Event.ADDED_TO_STAGE, _onAddedToStage);
		}
		
		public function isComplete():Boolean { return _isComplete; }
		public function hasFailed():Boolean { return _failed; }
		
		private function _onAddedToStage(e:Event):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, _onAddedToStage);
			addEventListener(MouseEvent.MOUSE_DOWN, _onMouseDown);
		}
		
		public function tick(tick:int):void 
		{
			var humans:uint = 0;
			var ais:uint = 0;
			const newOrgs:Vector.<Entity> = new Vector.<Entity>();
			for each (var e:Entity in _entities)
			{
				e.tick(tick);
				
				if (e is Organism)
				{
					const organism:Organism = e as Organism;
					if (organism.shouldSplit())
					{
						const no:Organism = organism.createChild();
						newOrgs.push(no);
						organism.resetSplit();
					}
					
					if (organism is HumanOrganism)
						++humans;
					else
						++ais;
				}
			}
			
			for each (var toAdd:Entity in newOrgs)
				add(toAdd);
			
			// remove
			for (var i:int = 0; i < _entities.length; ++i)
			{
				if (!_entities[i].canRemove())
					continue;
				removeChild(_entities[i]);
				if (i == _entities.length - 1)
					_entities.pop();
				else
					_entities[i--] = _entities.pop();
			}			
			
			if (ais == 0 && humans >= 5)
				_isComplete = true;
			else if (humans == 0)
				_failed = true;
				
			_totalOrgs = ais + humans;
		}
		
		public function render(tick:int, a:Number):void 
		{
			if (_selected != null && _selected.canRemove())
				deselect();
			
			if (_selected != null)
			{
				if (_selectionMarker.parent != null && _selectionMarker.parent != _selected)
					_selectionMarker.parent.removeChild(_selectionMarker);
					
				if (_selectionMarker.parent == null)
					_selected.addChild(_selectionMarker);
					
				_selectionMarker.setRadius(_selected.getRadius());
				_selectionMarker.render(tick, a);
			}
			
			for each (var e:Entity in _entities)
				e.render(tick, a);
		}		
		
		public function getFreePos(radius:Number):Vec2
		{
			while (true)
			{
				const p:Vec2 = new Vec2(Game.W * Math.random(), Game.H * Math.random());
				if (isFree(p, radius))
					return p;
			}
			return null;
		}
		
		public function all():Vector.<Entity> { return _entities; }

		public function isFree(pos:Vec2, radius:Number):Boolean
		{
			if (!isOnBoard(pos)) return false;
			for each (var e:Entity in _entities)
				if (e.overlaps(pos, radius))
					return false;
			return true;
		}
		
		public function isOnBoard(pos:Vec2):Boolean
		{
			if (pos.x < 30 || pos.x > Game.W-30) return false;
			if (pos.y < 30 || pos.y > Game.H-30) return false;
			return true;
		}
		
		public function getClosest(p:Vec2, r:Number, c:Class):Entity
		{
			var best:Entity = null
			var bestSqr:Number = Number.MAX_VALUE;
			const rsqr:Number = r * r;
			for each (var entity:entity.Entity in _entities)
			{
				if (!(entity is c))
					continue;
				
				const tmp:entity.Entity = entity as entity.Entity;
				if (!isOnBoard(tmp.getPos()))
					continue;
				
				const d:Number = tmp.getPos().distanceSqr(p);
				if (d > rsqr)
					continue;
				if (best == null || d < bestSqr)
				{
					best = entity;
					bestSqr = d;
				}
			}
			return best;
		}
		
		public function add(e:Entity):void 
		{
			addChild(e);
			_entities.push(e);
		}
		
		public function setSelected(o:HumanOrganism):void { _selected = o; }
		public function isSelected(o:HumanOrganism):Boolean { return _selected == o; }
		
		private function deselect():void 
		{
			if (_selected.contains(_selectionMarker))
				_selected.removeChild(_selectionMarker);
			_selectionMarker.visible = false;
			_selected = null;
		}
		
		private function _onMouseDown(e:MouseEvent):void 
		{
			if (_selected == null)
				return;
			const p:Vec2 = new Vec2(e.stageX, e.stageY);
			if (p.distance(_selected.getPos()) < _selected.getRadius() + 10)
				return;
			_selected.moveToCost(p);
		}
	}
}