package entity
{
	import entity.goal.Goal;
	import flash.display.Sprite;
	import util.Conv;
	import util.Vec2;
	/**
	 * ...
	 * @author 
	 */
	public class Entity extends Sprite
	{
		private const _pos:Vec2 = new Vec2();
		private const _dir:Vec2 = new Vec2(1, 0);
		private const _lastPos:Vec2 = new Vec2();
		private var _radius:Number = 1;
		private var _world:World;
		
		private const _dest:Vec2 = new Vec2;
		private const _goals:Vector.<Goal> = new Vector.<Goal>();

		private var _posDeltaRotation:Vec2;
		private var _negDeltaRotation:Vec2;
		private var _deviation:Number = 0;
		private var _canRemove:Boolean = false;
		private var _speedBonus:Number = 0;
		
		// tweakables
		private var _acc:Number = 0.01;
		private var _speed:Number = 0;
		private var _maxSpeed:Number = 0.1;
		private var _degPerTick:Number = 0;
		
		
		public function Entity(world:World, p:Vec2, radius:int) 
		{
			_world = world;
			_pos.copy(p);
			_dest.copy(p);
			setRadius(radius);
			setDegreesPerTick(1);
		}
		
		public function tick(tick:int):void
		{
			for each (var goal:Goal in _goals)
			{
				if (goal.canRun(tick))
				{
					goal.tick(tick);
					break;
				}
			}
			
			_lastPos.copy(_pos);
			
			_turn();
			_move();
			
			_speedBonus *= 0.97;
			
			if (_radius < 1.5)
				setToRemove();
		}
		
		public function render(tick:int, a:Number):void
		{
			const ip:Vec2 = _lastPos.lerp(_pos, a);
			this.x = ip.x;
			this.y = ip.y;
			this.rotation = _dir.getDegrees();
		}
				
		public function overlaps(p:Vec2, r:int):Boolean
		{
			const rtot:Number = _radius + r;
			return p.distanceSqr(_pos) <= rtot * rtot;
		}
		
		public function getPos():Vec2 { return _pos; }
		public function getDir():Vec2 { return _dir; }
		public function hasReached():Boolean { return _pos.distanceSqr(_dest) < 0.1; }
		public function addGoal(goal:Goal):void { _goals.push(goal); }
		public function setToRemove():void  { _canRemove = true; }
		public function canRemove():Boolean { return _canRemove; }
		public function getMaxSpeed():Number { return _maxSpeed; }
		public function getDegreesPerTick():Number  { return _degPerTick; }
		public function setMaxSpeed(s:Number):void { _maxSpeed = s; }
		public function getDest():Vec2 { return _dest; }
		public function addSpeedBonus(s:Number):void { _speedBonus += s; }
		public function getWorld():World { return _world; }
		public function setSpeed(s:Number):void { _speed = s; }
		public function setDir(d:Vec2):void { _dir.copy(d); }
		
		
		public function getRadius():Number { return _radius; }
		
		public function addRadius(delta:Number):void { setRadius(Math.max(1, _radius + delta)); }
		public function mulRadius(m:Number):void { setRadius(Math.max(1, _radius * m)); }
		public function setRadius(r:Number):void
		{ 
			_radius = Math.max(1, r);
		}
		
		
		// TURN stuff
		public function setDegreesPerTick(degreesPerTick:Number):void
		{
			_degPerTick = degreesPerTick;
			_posDeltaRotation =  new Vec2(1, 0).rotate(-Conv.deg2rad(degreesPerTick));
			_negDeltaRotation =  new Vec2(1, 0).rotate(Conv.deg2rad(degreesPerTick));
			_deviation = Math.cos(Conv.deg2rad(degreesPerTick + 1));
		}
		
		private const _tmp:Vec2 = new Vec2;
		
		private function _turn():void
		{
			if (hasReached())
				return;
			
			_tmp.copy(_dest).subSelf(_pos);
			if (_tmp.lengthSqr() < 0.000001)
				_tmp.copyXY(1, 0);
			_tmp.normalizeSelf();
			if (_dir.dot(_tmp) < _deviation)
			{
				const rotation:Vec2 = _dir.crossDet(_tmp) > 0 ? _negDeltaRotation : _posDeltaRotation;
				_dir.rotateSpinorSelf(rotation).normalizeSelf();
				return;
			}
			_dir.copy(_tmp);
		}
		
		// MOVE stuff
		public function moveTo(dest:Vec2):void 
		{
			_dest.copy(dest);
		}
		
		private function _move():void 
		{
			_speed = Math.min(_maxSpeed + _speedBonus, _speed + _acc);
			
			const left:Number = _dest.distance(_pos);
			const s:Number = Math.min(left, _speed);
			_tmp.copy(_dir).rescaleSelf(s);
			_pos.addSelf(_tmp);
			
			if (hasReached())
			{
				_pos.copy(_dest);
				return;
			}
		}
	}
}