package entity 
{
	import com.greensock.TweenLite;
	import flash.display.Graphics;
	import flash.display.LineScaleMode;
	import flash.display.Sprite;
	import sound.Audio;
	import util.Vec2;
	/**
	 * ...
	 * @author 
	 */
	public class Organism extends Entity 
	{
		private var _split:Boolean = false;
		protected var _bodyColor:uint;
		protected var _ringColor:uint;
		protected var _cellColor:uint;
		private const _body:Sprite = new Sprite;
		private const _innerBodies:Vector.<Sprite> = new Vector.<Sprite>();
		
		// tweakables
		private var metabolism:Number = 0.8;
		private var _glucose:Number = 0;
		
		private static const _MaxRadius:Number = 50;
		private var _lastTime:Number = 0;
		
		private var _lastFoodTick:int = 0;
		
		public function Organism(world:World, p:Vec2, 
			bodyColor:uint,
			ringColor:uint,
			cellColor:uint)
		{
			super(world, p, 10);
			_bodyColor = bodyColor;
			_ringColor = ringColor;
			_cellColor = cellColor;
			
			_draw();
			
			addChild(_body);
			
			setMaxSpeed(0.1);
			setDegreesPerTick(4);
		}
		
		private function _draw():void 
		{
			const b:Graphics = _body.graphics;
			// body
			b.beginFill(_bodyColor, 0.4);
			b.drawCircle(0, 0, 1);
			b.endFill();
			
			// ring
			b.lineStyle(2, _ringColor, 0.5, false, LineScaleMode.NONE);
			b.drawCircle(0, 0, 1);
			
			_createCell(_cellColor);
			_createCell(_cellColor);
			_createCell(_cellColor);
		}
		
		private function _createCell(color:uint):Sprite
		{
			const xx:Number = 0.3 * (Math.random() * 2 - 1)
			const yy:Number = 0.3 * (Math.random() * 2 - 1)
			const r:Number = 0.2 + Math.random() * 0.3;
			const s:Sprite = new Sprite;
			const g:Graphics = s.graphics;
			g.beginFill(color, 0.9);
			g.drawCircle(xx, yy, r);
			g.endFill();
			_body.addChild(s);
			_innerBodies.push(s);
			return s;
		}
		
		public function createChild():Organism { return null;  }
		public function resetSplit():void  { _split = false; }
		public function shouldSplit():Boolean { return _split; }
		public function addMetabolism(delta:Number):void  { metabolism = Math.min(2, metabolism + delta); }
		public override function setMaxSpeed(s:Number):void { super.setMaxSpeed(Math.min(1, getMaxSpeed() + s)); }
		public function isSameTeam(o:Organism):Boolean { return _bodyColor == o._bodyColor; }
		public function getColor():uint { return _bodyColor; }
		public function getLeft():Number { return getPos().distance(getDest()); }
		public override function setRadius(r:Number):void 
		{
			TweenLite.to(_body, 0.5, { scaleX:r, scaleY:r } );
			super.setRadius(r);
		}
		
		public override function tick(tick:int):void
		{
			_body.alpha = 0.9;
			
			const entities:Vector.<Entity> = getWorld().all();
			for each (var e:Entity in entities)
			{
				if (e is Food)
				{
					if (!e.overlaps(getPos(), getRadius()))
						continue;
					if ((e as Food).canEat())
					{
						_consume(e);
						_lastFoodTick = tick;
					}
				}
				else if (e is Booster)
				{
					if (!e.overlaps(getPos(), getRadius()))
						continue;
					const booster:Booster = e as Booster;
					booster.applyTo(this);
					booster.setToRemove();
					Audio.playBooster();
				}
				else if (e is Poison)
				{
					if (!e.overlaps(getPos(), getRadius()))
						continue;
					e.setToRemove();
					_removeEnergy(e.getRadius() * metabolism);
					Audio.playPoison();
				}
				else if (e is Organism)
				{
					if (e.canRemove())
						continue;
					if (!e.overlaps(getPos(), getRadius()))
						continue;
					const organism:Organism = e as Organism;
					if (isSameTeam(organism))
						continue;
					if (getRadius() > organism.getRadius())
						_consume(organism);
					else
						organism._consume(this);
				}
			}
			
			//if (tick - _lastFoodTick > 20 * 4)
			//{
				//mulRadius(0.97);
				//_lastFoodTick = tick;
			//}
			
			if (_glucose >= 1)
				_splitIt();
				
			super.tick(tick);
		}
		
		public function moveToCost(dest:Vec2):void
		{
			addSpeedBonus(1.5);
			mulRadius(0.97);
			_glucose *= 0.97;
			super.moveTo(dest);
			
			const no:Food = new Food(getWorld(), getPos(), Math.min(3, getRadius()));
			const d:Vec2 = getPos().sub(getDest());
			if (d.lengthSqr() < 0.000001)
				d.copyXY(1, 0);
			d.rescaleSelf(40 + 20 * Math.random());
			
			no.moveTo(getPos().add(d));
			no.setDir(d.normalize());
			no.addSpeedBonus(4 + 5 * Math.random());
			no.setSpeed(2 + 2 * Math.random());
			getWorld().add(no);
			
			if (this is HumanOrganism)
				Audio.playFart();
			else if (Math.random() > 0.8)
				Audio.playFart();
		}
		
		private function _consume(e:Entity):void 
		{
			e.setToRemove();
			_addEnergy(e.getRadius() * metabolism);
			
			if (e is Organism)
				Audio.playDie();
			else if (this is HumanOrganism || Math.random() > 0.8)
				Audio.playEat();
		}
		
		public override function render(tick:int, a:Number):void
		{
			const now:Number = tick + a;
			for (var i:int = 0; i < _innerBodies.length; ++i)
			{
				_innerBodies[i].rotation += (now - _lastTime) / (1 + i);
				_innerBodies[i].alpha = 0.3 + 0.7 * _glucose;
			}
			_lastTime = now;
			
			super.render(tick, a);
		}
		
		private function _addEnergy(r:Number):void
		{
			const tr:Number = getRadius();
			setRadius(Math.max(0.1, Math.min(_MaxRadius, Math.sqrt(tr * tr + r * r))));
			_glucose = Math.max(0, Math.min(1, _glucose + 0.2));
		}
		
		private function _removeEnergy(r:Number):void 
		{
			const tr:Number = getRadius();
			setRadius(Math.min(_MaxRadius, Math.sqrt(Math.max(tr * tr - r * r, 1))));
			_glucose = Math.max(0, Math.min(1, _glucose - 0.2));
		}
		
		protected function _splitIt():void 
		{
			_glucose = 0;
			mulRadius(0.98);
			if (getWorld()._totalOrgs > 35)
				return;
			
			_split = true;
			Audio.playSplit();
		}
	}
}